@@ -1493,7 +1493,6 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 	g_free (val);
 
 	e_table->sorter = e_table_sorter_new(etm, e_table->full_header, e_table->sort_info);
-	g_object_ref (e_table->sorter);
 
 	g_object_set (e_table->selection,
 		      "model", etm,
@@ -2760,7 +2759,7 @@ do_drag_motion(ETable *et,
 	       guint time)
 {
 	gboolean ret_val;
-	int row, col;
+	int row = -1, col = -1;
 	GtkWidget *widget;
 
 	widget = GTK_WIDGET (et);
@@ -3142,7 +3141,7 @@ e_table_class_init (ETableClass *class)
 			      G_TYPE_INT,
 			      G_TYPE_INT,
 			      GDK_TYPE_DRAG_CONTEXT,
-			      GTK_TYPE_SELECTION_DATA,
+			      GTK_TYPE_SELECTION_DATA | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      G_TYPE_UINT,
 			      G_TYPE_UINT);
 	et_signals[TABLE_DRAG_DATA_DELETE] =
@@ -3210,7 +3209,7 @@ e_table_class_init (ETableClass *class)
 			      GDK_TYPE_DRAG_CONTEXT,
 			      G_TYPE_INT,
 			      G_TYPE_INT,
-			      GTK_TYPE_SELECTION_DATA,
+			      GTK_TYPE_SELECTION_DATA | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      G_TYPE_UINT,
 			      G_TYPE_UINT);
 
