@@ -399,6 +399,7 @@ insert_node(ETreeTableAdapter *etta, ETreePath parent, ETreePath path)
 			generate_tree(etta, parent);
 		else
 			insert_node(etta, grandparent, parent);
+		e_table_model_changed(E_TABLE_MODEL(etta));
 		return;
 	}
 
@@ -473,7 +474,7 @@ update_node(ETreeTableAdapter *etta, ETreePath path)
 	closure.paths = NULL;
 
 	if (gnode)
-		g_node_traverse(gnode, G_IN_ORDER, G_TRAVERSE_ALL, -1, check_expanded, &closure);
+		g_node_traverse(gnode, G_POST_ORDER, G_TRAVERSE_ALL, -1, check_expanded, &closure);
 
 	if (e_tree_model_node_is_root(etta->priv->source, path))
 		generate_tree(etta, path);
@@ -483,7 +484,8 @@ update_node(ETreeTableAdapter *etta, ETreePath path)
 	}
 
 	for (l = closure.paths; l; l = l->next)
-		e_tree_table_adapter_node_set_expanded (etta, l->data, !closure.expanded);
+		if (lookup_gnode(etta, l->data))
+			e_tree_table_adapter_node_set_expanded (etta, l->data, !closure.expanded);
 
 	g_slist_free(closure.paths);
 }
@@ -782,12 +784,15 @@ etta_proxy_node_inserted (ETreeModel *etm, ETreePath parent, ETreePath child, ET
 		generate_tree(etta, child);
 	else
 		insert_node(etta, parent, child);
+	
+	e_table_model_changed(E_TABLE_MODEL(etta));
 }
 
 static void
 etta_proxy_node_removed (ETreeModel *etm, ETreePath parent, ETreePath child, int old_position, ETreeTableAdapter *etta)
 {
 	delete_node(etta, parent, child);
+	e_table_model_changed(E_TABLE_MODEL(etta));
 }
 
 static void
