@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * color-palette.c - A color selector palette
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -31,8 +31,9 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtktable.h>
-#include <libgnomeui/gnome-canvas.h>
-#include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include <libgnomecanvas/gnome-canvas.h>
+#include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
+#include <gdk/gdkcolor.h>
 #include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
 #include "color-group.h"
@@ -43,128 +44,101 @@
 #define COLOR_PREVIEW_HEIGHT 15
 
 enum {
-	CHANGED,
+	COLOR_CHANGED,
 	LAST_SIGNAL
 };
 
 struct _ColorNamePair {
-	char *color;	/* rgb color or otherwise - eg. "rgb:FF/FF/FF" */
+	char *color;	/* rgb color or otherwise - eg. "#FFFFFF" */
 	char *name;	/* english name - eg. "white" */
 };
 
-static gint color_palette_signals [LAST_SIGNAL] = { 0, };
+static guint color_palette_signals [LAST_SIGNAL] = { 0, };
 
-static GtkObjectClass *color_palette_parent_class;
+#define PARENT_TYPE GTK_TYPE_VBOX
+static GObjectClass *color_palette_parent_class;
 
 #define make_color(P,COL) (((COL) != NULL) ? (COL) : ((P) ? ((P)->default_color) : NULL))
 
 static void
-color_palette_finalize (GtkObject *object)
+color_palette_destroy (GtkObject *object)
 {
 	ColorPalette *P = COLOR_PALETTE (object);
+	GtkObjectClass *klass = (GtkObjectClass *)color_palette_parent_class;
 
 	if (P->tool_tip) {
-		gtk_object_unref (GTK_OBJECT (P->tool_tip));
+		g_object_unref (P->tool_tip);
 		P->tool_tip = NULL;
 	}
 
-	g_free (P->items);
-
-	if (P->current_color)
+	if (P->current_color) {
 		gdk_color_free (P->current_color);
+		P->current_color = NULL;
+	}
 	
 	color_palette_set_group (P, NULL);
 
+	memset (P->items, 0, P->total * sizeof (GnomeCanvasItem *));
+
+	if (klass->destroy)
+                klass->destroy (object);
+}
+
+static void
+color_palette_finalize (GObject *object)
+{
+	ColorPalette *P = COLOR_PALETTE (object);
+
+	g_free (P->items);
+
 	(*color_palette_parent_class->finalize) (object);
 }
 
-
-typedef void (*GtkSignal_NONE__POINTER_BOOL_BOOL) (GtkObject * object,
-						   gpointer arg1,
-						   gboolean arg2,
-						   gboolean arg3,
-						   gpointer user_data);
 static void
-marshal_NONE__POINTER_BOOL_BOOL (GtkObject * object,
-				 GtkSignalFunc func,
-				 gpointer func_data,
-				 GtkArg * args)
-{
-	GtkSignal_NONE__POINTER_BOOL_BOOL rfunc;
-	rfunc = (GtkSignal_NONE__POINTER_BOOL_BOOL) func;
-	(*rfunc) (object,
-		  GTK_VALUE_POINTER (args[0]),
-		  GTK_VALUE_BOOL    (args[1]),
-		  GTK_VALUE_BOOL    (args[2]),
-		  func_data);
-}
-
-
-static void
-color_palette_class_init (GtkObjectClass *object_class)
+color_palette_class_init (GObjectClass *object_class)
 {
 	object_class->finalize = color_palette_finalize;
+	((GtkObjectClass *)object_class)->destroy = color_palette_destroy;
 
-	color_palette_parent_class = gtk_type_class (gtk_vbox_get_type ());
+	color_palette_parent_class = g_type_class_peek_parent (object_class);
 
-	color_palette_signals [CHANGED] =
-		gtk_signal_new (
-			"changed",
-			GTK_RUN_LAST,
-			E_OBJECT_CLASS_TYPE (object_class),
-			GTK_SIGNAL_OFFSET (ColorPaletteClass, changed),
-			marshal_NONE__POINTER_BOOL_BOOL,
-			GTK_TYPE_NONE, 3, GTK_TYPE_POINTER,
-			GTK_TYPE_BOOL, GTK_TYPE_BOOL);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, color_palette_signals, LAST_SIGNAL);
+	color_palette_signals [COLOR_CHANGED] =
+		g_signal_new ("color_changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ColorPaletteClass, color_changed),
+			      NULL, NULL,
+			      e_marshal_NONE__POINTER_BOOLEAN_BOOLEAN_BOOLEAN,
+			      G_TYPE_NONE, 4, G_TYPE_POINTER,
+			      G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
 }
 
-GtkType
-color_palette_get_type (void)
-{
-	static GtkType type = 0;
+E_MAKE_TYPE (color_palette,
+	     "ColorPalette",
+	     ColorPalette,
+	     color_palette_class_init,
+	     NULL,
+	     PARENT_TYPE)
 
-	if (!type){
-		GtkTypeInfo info = {
-			"ColorPalette",
-			sizeof (ColorPalette),
-			sizeof (ColorPaletteClass),
-			(GtkClassInitFunc) color_palette_class_init,
-			(GtkObjectInitFunc) NULL,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (gtk_vbox_get_type (), &info);
-	}
-
-	return type;
-}
-
-/*
- * Fires signal "changed" with color as its param
- *
- * prop
- */
 static void
-emit_change (ColorPalette *P, GdkColor *color, gboolean custom, gboolean by_user)
+emit_color_changed (ColorPalette *P, GdkColor *color,
+		    gboolean custom, gboolean by_user, gboolean is_default)
 {
-	GdkColor *new;
-	
-	/* Set it as current color */
+	GdkColor *new = make_color (P, color);
+
+	if (new != NULL)
+		new = gdk_color_copy (new);
 	if (P->current_color)
 		gdk_color_free (P->current_color);
-	new = make_color (P, color);
-	P->current_color = new ? gdk_color_copy (new) : NULL;
+	P->current_color = new;
+	P->current_is_default = is_default;
 
 	/* Only add custom colors to the group */
 	if (custom && color)
 		color_group_add_color (P->color_group, color);
 	
-	gtk_signal_emit (
-		GTK_OBJECT (P), color_palette_signals [CHANGED], color, custom, by_user);
+	g_signal_emit (P, color_palette_signals [COLOR_CHANGED], 0,
+		       color, custom, by_user, is_default);
 }
 
 
@@ -192,21 +166,21 @@ color_palette_change_custom_color (ColorPalette *P, GdkColor const * const new)
 	for (index = P->custom_color_pos; index < P->total - 1; index++) {
 		GdkColor *color;
 		GdkColor *outline;
-		item = P->items [index];
-		next_item = P->items [index + 1];
+		item = P->items[index];
+		next_item = P->items[index + 1];
 
-		gtk_object_get (GTK_OBJECT (next_item),
-				"fill_color_gdk", &color,
-				"outline_color_gdk", &outline,
-				NULL);
+		g_object_get (G_OBJECT (next_item),
+			"fill_color_gdk",	&color,
+			"outline_color_gdk",	&outline,
+			NULL);
 		gnome_canvas_item_set (item,
-				       "fill_color_gdk", color,
-				       "outline_color_gdk", outline,
-				       NULL);
-		g_free (color);
-		g_free (outline);
+			"fill_color_gdk",	color,
+			"outline_color_gdk",	outline,
+			NULL);
+		gdk_color_free (color);
+		gdk_color_free (outline);
 	}
-	item = P->items [index];
+	item = P->items[index];
 	gnome_canvas_item_set (item,
 			       "fill_color_gdk", new,
 			       "outline_color_gdk", new,
@@ -216,7 +190,6 @@ color_palette_change_custom_color (ColorPalette *P, GdkColor const * const new)
 				    new->green,
 				    new->blue,
 				    0);
-	return;
 }
 
 /*
@@ -227,27 +200,20 @@ static void
 cust_color_set (GtkWidget  *color_picker, guint r, guint g, guint b, guint a,
 		ColorPalette *P)
 {
-	GdkColor *c_color;
+	GdkColor c_color;
 
-	c_color = g_new (GdkColor, 1);
+	c_color.red   = (gushort)r;
+	c_color.green = (gushort)g;
+	c_color.blue  = (gushort)b;
 
-	c_color->red   = (gushort)r;
-	c_color->green = (gushort)g;
-	c_color->blue  = (gushort)b;
-
-	e_color_alloc_gdk (c_color);
-	emit_change (P, c_color, TRUE, TRUE);
-	g_free (c_color);
+	e_color_alloc_gdk (NULL, &c_color);
+	emit_color_changed (P, &c_color, TRUE, TRUE, FALSE);
 }
 
-/*
- * NoColour/Auto button was pressed.
- * emit a signal with 'NULL' color.
- */
 static void
-cb_nocolor_clicked (GtkWidget *button, ColorPalette *P)
+cb_default_clicked (GtkWidget *button, ColorPalette *P)
 {
-        emit_change (P, P->default_color, FALSE, TRUE);
+	emit_color_changed (P, P->default_color, FALSE, TRUE, TRUE);
 }
 
 /*
@@ -261,16 +227,15 @@ color_clicked (GtkWidget *button, ColorPalette *P)
 	GdkColor        *gdk_color;
 
 	index = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (button)));
-	item  = P->items [index];
+	item  = P->items[index];
 
-	gtk_object_get (
-		GTK_OBJECT (item),
-		"fill_color_gdk", &gdk_color,
-		NULL);
+	g_object_get (item,
+		      "fill_color_gdk", &gdk_color,
+		      NULL);
 
-	emit_change (P, gdk_color, FALSE, TRUE);
+	emit_color_changed (P, gdk_color, FALSE, TRUE, FALSE);
 
-	g_free (gdk_color);
+	gdk_color_free (gdk_color);
 }
 
 /*
@@ -332,11 +297,7 @@ color_palette_button_new(ColorPalette *P, GtkTable* table,
 	button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 
-	gtk_widget_push_visual (gdk_imlib_get_visual ());
-	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 	canvas = gnome_canvas_new ();
-	gtk_widget_pop_colormap ();
-	gtk_widget_pop_visual ();
 
 	gtk_widget_set_usize (canvas, COLOR_PREVIEW_WIDTH, COLOR_PREVIEW_HEIGHT);
 	gtk_container_add (GTK_CONTAINER (button), canvas);
@@ -357,8 +318,8 @@ color_palette_button_new(ColorPalette *P, GtkTable* table,
 	gtk_table_attach (table, button,
 			  col, col+1, row, row+1, GTK_FILL, GTK_FILL, 1, 1);
 
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC(color_clicked), P);
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (color_clicked), P);
 	gtk_object_set_user_data (GTK_OBJECT (button),
 				  GINT_TO_POINTER (data));
 	return item;
@@ -395,7 +356,7 @@ color_palette_setup (ColorPalette *P,
 		     int ncols, int nrows,
 		     ColorNamePair *color_names)
 {
-	GtkWidget *nocolor_button;
+	GtkWidget *default_button;
 	GtkWidget *cust_label;
 	GtkWidget *table;
 	GtkTooltips *tool_tip;
@@ -403,17 +364,19 @@ color_palette_setup (ColorPalette *P,
 
 	table = gtk_table_new (ncols, nrows, FALSE);
 
-
 	if (no_color_label != NULL) {
-		nocolor_button = gtk_button_new_with_label (no_color_label);
+		default_button = gtk_button_new_with_label (no_color_label);
 
-		gtk_table_attach (GTK_TABLE (table), nocolor_button,
+		gtk_table_attach (GTK_TABLE (table), default_button,
 				  0, ncols, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
-		gtk_signal_connect (GTK_OBJECT (nocolor_button), "clicked",
-				    GTK_SIGNAL_FUNC(cb_nocolor_clicked), P);
+		g_signal_connect (default_button, "clicked",
+				  G_CALLBACK (cb_default_clicked), P);
 	}
 
-	P->tool_tip = tool_tip = gtk_tooltips_new();
+	P->tool_tip = tool_tip = gtk_tooltips_new ();
+	g_object_ref (P->tool_tip);
+	gtk_object_sink (GTK_OBJECT (P->tool_tip));
+
 	P->custom_color_pos = -1;
 	total = 0;
 
@@ -427,7 +390,7 @@ color_palette_setup (ColorPalette *P,
 			 */
 			if (color_names [pos].color == NULL) {
 				/* This is the default custom color */
-				ColorNamePair color_name  = {"rgb:0/0/0", N_("custom")};
+				ColorNamePair color_name  = {"#000", N_("custom")};
 				row++;
 				if (col == 0 || row < nrows) {
 					/* Add a full row for custom colors */
@@ -453,7 +416,7 @@ color_palette_setup (ColorPalette *P,
 				break;
 			}
 
-			P->items [total] =
+			P->items[total] =
 				color_palette_button_new (
 					P,
 					GTK_TABLE (table),
@@ -480,26 +443,43 @@ color_palette_setup (ColorPalette *P,
 	gnome_color_picker_set_title (P->picker, _("Choose Custom Color"));
 	gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (P->picker), ncols - 3, ncols,
 			  row + 1, row + 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
-	gtk_signal_connect (GTK_OBJECT (P->picker), "color_set",
-			    GTK_SIGNAL_FUNC (cust_color_set), P);
+	g_signal_connect (P->picker, "color_set",
+			  G_CALLBACK (cust_color_set), P);
 	return table;
 }
 
+void
+color_palette_set_color_to_default (ColorPalette *P)
+{
+	g_return_if_fail (P != NULL);
+	g_return_if_fail (IS_COLOR_GROUP (P->color_group));
+
+	emit_color_changed (P, P->default_color, FALSE, TRUE, TRUE);
+}
+
 void
 color_palette_set_current_color (ColorPalette *P, GdkColor *color)
 {
 	g_return_if_fail (P != NULL);
 	g_return_if_fail (IS_COLOR_GROUP (P->color_group));
 
-	emit_change (P, color, color_in_palette (P->default_set, color), FALSE);
+	if (color)
+		emit_color_changed
+			(P, color, color_in_palette (P->default_set, color),
+			 FALSE, FALSE);
+	else
+		color_palette_set_color_to_default (P);
 }
 
 GdkColor *
-color_palette_get_current_color(ColorPalette *P)
+color_palette_get_current_color (ColorPalette *P, gboolean *is_default)
 {
 	g_return_val_if_fail (P != NULL, NULL);
 	g_return_val_if_fail (IS_COLOR_GROUP (P->color_group), NULL);
 
+	if (is_default != NULL)
+		*is_default = P->current_is_default;
+
 	return P->current_color ? gdk_color_copy (P->current_color) : NULL;
 }
 
@@ -552,11 +532,12 @@ color_palette_new_with_vals (char const * const no_color_label,
 
 	g_return_val_if_fail (color_names != NULL, NULL);
 
-	P = gtk_type_new (color_palette_get_type ());
+	P = g_object_new (COLOR_PALETTE_TYPE, NULL);
 
 	P->default_set   = color_names;
 	P->default_color = default_color;
 	P->current_color = default_color ? gdk_color_copy (default_color) : NULL;
+	P->current_is_default = TRUE;
 	color_palette_set_group (P, cg);
 
 	color_palette_construct (P, no_color_label, ncols, nrows);
@@ -565,6 +546,9 @@ color_palette_new_with_vals (char const * const no_color_label,
 	return GTK_WIDGET (P);
 }
 
+/**
+ * color_palette_set_group : absorbs the reference to the group
+ */
 void
 color_palette_set_group (ColorPalette *P, ColorGroup *cg)
 {
@@ -572,92 +556,92 @@ color_palette_set_group (ColorPalette *P, ColorGroup *cg)
 		return;
 
 	if (P->color_group) {
-		gtk_signal_disconnect_by_func (
-			GTK_OBJECT (P->color_group),
-			GTK_SIGNAL_FUNC (cb_group_custom_color_add),
+		g_signal_handlers_disconnect_by_func (
+			G_OBJECT (P->color_group),
+			G_CALLBACK (cb_group_custom_color_add),
 			P);
-		gtk_object_unref(GTK_OBJECT (P->color_group));
+		g_object_unref (G_OBJECT (P->color_group));
 		P->color_group = NULL;
 	}
 	if (cg != NULL) {
 		P->color_group = COLOR_GROUP (cg);
-		gtk_signal_connect (GTK_OBJECT (cg), "custom_color_add",
-			GTK_SIGNAL_FUNC (cb_group_custom_color_add),
+		g_signal_connect (G_OBJECT (cg), "custom_color_add",
+			G_CALLBACK (cb_group_custom_color_add),
 			P);
 
 	}
 }
 
 static ColorNamePair default_color_set [] = {
-	{"rgb:0/0/0", N_("black")},
-	{"rgb:99/33/0", N_("light brown")},
-	{"rgb:33/33/0", N_("brown gold")},
-	{"rgb:0/33/0", N_("dark green #2")},
-	{"rgb:0/33/66", N_("navy")},
-	{"rgb:0/0/80", N_("dark blue")},
-	{"rgb:33/33/99", N_("purple #2")},
-	{"rgb:33/33/33", N_("very dark gray")},
+	{"#000000", N_("black")},
+	{"#993300", N_("light brown")},
+	{"#333300", N_("brown gold")},
+	{"#003300", N_("dark green #2")},
+	{"#003366", N_("navy")},
+	{"#000080", N_("dark blue")},
+	{"#333399", N_("purple #2")},
+	{"#333333", N_("very dark gray")},
 
 
-	{"rgb:80/0/0", N_("dark red")},
-	{"rgb:FF/66/0", N_("red-orange")},
-	{"rgb:80/80/0", N_("gold")},
-	{"rgb:0/80/0", N_("dark green")},
-	{"rgb:0/80/80", N_("dull blue")},
-	{"rgb:0/0/FF", N_("blue")},
-	{"rgb:66/66/99", N_("dull purple")},
-	{"rgb:80/80/80", N_("dark grey")},
+	{"#800000", N_("dark red")},
+	{"#FF6600", N_("red-orange")},
+	{"#808000", N_("gold")},
+	{"#008000", N_("dark green")},
+	{"#008080", N_("dull blue")},
+	{"#0000FF", N_("blue")},
+	{"#666699", N_("dull purple")},
+	{"#808080", N_("dark grey")},
 
 
-	{"rgb:FF/0/0", N_("red")},
-	{"rgb:FF/99/0", N_("orange")},
-	{"rgb:99/CC/0", N_("lime")},
-	{"rgb:33/99/66", N_("dull green")},
-	{"rgb:33/CC/CC",N_("dull blue #2")},
-	{"rgb:33/66/FF", N_("sky blue #2")},
-	{"rgb:80/0/80", N_("purple")},
-	{"rgb:96/96/96", N_("gray")},
+	{"#FF0000", N_("red")},
+	{"#FF9900", N_("orange")},
+	{"#99CC00", N_("lime")},
+	{"#339966", N_("dull green")},
+	{"#33CCCC",N_("dull blue #2")},
+	{"#3366FF", N_("sky blue #2")},
+	{"#800080", N_("purple")},
+	{"#969696", N_("gray")},
 
 
-	{"rgb:FF/0/FF", N_("magenta")},
-	{"rgb:FF/CC/0", N_("bright orange")},
-	{"rgb:FF/FF/0", N_("yellow")},
-	{"rgb:0/FF/0", N_("green")},
-	{"rgb:0/FF/FF", N_("cyan")},
-	{"rgb:0/CC/FF", N_("bright blue")},
-	{"rgb:99/33/66", N_("red purple")},
-	{"rgb:c0/c0/c0", N_("light grey")},
+	{"#FF00FF", N_("magenta")},
+	{"#FFCC00", N_("bright orange")},
+	{"#FFFF00", N_("yellow")},
+	{"#00FF00", N_("green")},
+	{"#00FFFF", N_("cyan")},
+	{"#00CCFF", N_("bright blue")},
+	{"#993366", N_("red purple")},
+	{"#C0C0C0", N_("light grey")},
 
 
-	{"rgb:FF/99/CC", N_("pink")},
-	{"rgb:FF/CC/99", N_("light orange")},
-	{"rgb:FF/FF/99", N_("light yellow")},
-	{"rgb:CC/FF/CC", N_("light green")},
-	{"rgb:CC/FF/FF", N_("light cyan")},
-	{"rgb:99/CC/FF", N_("light blue")},
-	{"rgb:CC/99/FF", N_("light purple")},
-	{"rgb:FF/FF/FF", N_("white")},
+	{"#FF99CC", N_("pink")},
+	{"#FFCC99", N_("light orange")},
+	{"#FFFF99", N_("light yellow")},
+	{"#CCFFCC", N_("light green")},
+	{"#CCFFFF", N_("light cyan")},
+	{"#99CCFF", N_("light blue")},
+	{"#CC99FF", N_("light purple")},
+	{"#FFFFFF", N_("white")},
 
 	/* Disable these for now, they are mostly repeats */
 	{NULL, NULL},
 
-	{"rgb:99/99/FF", N_("purplish blue")},
-	{"rgb:99/33/66", N_("red purple")},
-	{"rgb:FF/FF/CC", N_("light yellow")},
-	{"rgb:CC/FF/FF", N_("light blue")},
-	{"rgb:66/0/66", N_("dark purple")},
-	{"rgb:FF/80/80", N_("pink")},
-	{"rgb:0/66/CC", N_("sky blue")},
-	{"rgb:CC/CC/FF", N_("light purple")},
+	{"#9999FF", N_("purplish blue")},
+	{"#993366", N_("red purple")},
+	{"#FFFFCC", N_("light yellow")},
+	{"#CCFFFF", N_("light blue")},
+	{"#660066", N_("dark purple")},
+	{"#FF8080", N_("pink")},
+	{"#0066CC", N_("sky blue")},
+	{"#CCCCFF", N_("light purple")},
 
-	{"rgb:0/0/80", N_("dark blue")},
-	{"rgb:FF/0/FF", N_("magenta")},
-	{"rgb:FF/FF/0", N_("yellow")},
-	{"rgb:0/FF/FF", N_("cyan")},
-	{"rgb:80/0/80", N_("purple")},
-	{"rgb:80/0/0", N_("dark red")},
-	{"rgb:0/80/80", N_("dull blue")},
-	{"rgb:0/0/FF", N_("blue")},
+	{"#000080", N_("dark blue")},
+	{"#FF00FF", N_("magenta")},
+	{"#FFFF00", N_("yellow")},
+	{"#00FFFF", N_("cyan")},
+	{"#800080", N_("purple")},
+	{"#800000", N_("dark red")},
+	{"#008080", N_("dull blue")},
+	{"#0000FF", N_("blue")},
 
 	{NULL, NULL}
 };
